@@ -1,9 +1,9 @@
 
-__version__ = "0.1.5"
+__version__ = "0.3.1"
 __common_alias__ = "nppas"
 
 from napari.types import SurfaceData, PointsData
-from napari.types import LabelsData, LayerData
+from napari.types import LabelsData, ImageData
 
 from napari_plugin_engine import napari_hook_implementation
 from napari_tools_menu import register_function, register_action
@@ -41,18 +41,18 @@ def _knot_mesh() -> SurfaceData:
     import open3d
     from pathlib import Path
     data = str(Path(__file__).parent / "data" / "knot.ply")
-    return to_surface(open3d.io.read_triangle_mesh(data))
+    return isotropic_scale_surface(to_surface(open3d.io.read_triangle_mesh(data)), 0.1)
 
 def _standford_bunny() -> SurfaceData:
     import open3d
     from pathlib import Path
     data = str(Path(__file__).parent / "data" / "bun_zipper.ply")
-    return to_surface(open3d.io.read_triangle_mesh(data))
+    return isotropic_scale_surface(to_surface(open3d.io.read_triangle_mesh(data)), 100)
 
 def _vedo_ellipsoid() -> SurfaceData:
     import vedo
     shape = vedo.shapes.Ellipsoid()
-    return (shape.points(), np.asarray(shape.faces()))
+    return isotropic_scale_surface((shape.points(), np.asarray(shape.faces())), 10)
 
 @register_action(menu = "Surfaces > Example data: Knot (open3d, nppas)")
 def example_data_knot(viewer:napari.viewer):
@@ -107,14 +107,6 @@ def to_point_cloud(data):
     return pcd
 
 
-def return_points(data, viewer: napari.Viewer = None):
-    if viewer is not None:
-        # This is a workaround, because I don't know how to pass point size back as result
-        viewer.add_points(data, size=0.5, edge_color="white")
-    else:
-        return data
-
-
 def to_surface(mesh):
     vertices = to_numpy(mesh.vertices)
     faces = to_numpy(mesh.triangles)
@@ -124,7 +116,7 @@ def to_surface(mesh):
 
 
 @register_function(menu="Surfaces > Convex hull (open3d, nppas)")
-def convex_hull(surface):
+def convex_hull(surface:SurfaceData) -> SurfaceData:
     """Produce the convex hull surface around a surface
     """
     mesh = to_mesh(surface)
@@ -248,8 +240,8 @@ def subdivide_loop(surface:SurfaceData, number_of_iterations: int = 1) -> Surfac
     return to_surface(mesh_out)
 
 
-@register_function(menu="Points > Labels to centroids (nppas)")
-def labels_to_centroids(labels_data:LabelsData, viewer:napari.Viewer = None):
+@register_function(menu="Points > Create points from labels centroids (nppas)")
+def labels_to_centroids(labels_data:LabelsData, viewer:napari.Viewer = None) -> PointsData:
     """Determine centroids from all labels and store them as points.
 
     Parameters
@@ -260,11 +252,11 @@ def labels_to_centroids(labels_data:LabelsData, viewer:napari.Viewer = None):
 
     statistics = regionprops(labels_data)
     centroids = [s.centroid for s in statistics]
-    return return_points(centroids, viewer)
+    return centroids
 
 
-@register_function(menu="Points > Sample from surface uniformly (open3d, nppas)")
-def sample_points_uniformly(surface:SurfaceData, number_of_points: int = 500, viewer:napari.Viewer=None):
+@register_function(menu="Points > Create points from surface sampling uniformly (open3d, nppas)")
+def sample_points_uniformly(surface:SurfaceData, number_of_points: int = 500, viewer:napari.Viewer=None) -> PointsData:
     """Sample points uniformly
 
     Parameters
@@ -280,11 +272,11 @@ def sample_points_uniformly(surface:SurfaceData, number_of_points: int = 500, vi
     point_cloud = mesh_in.sample_points_uniformly(number_of_points=number_of_points)
 
     result = to_numpy(point_cloud.points)
-    return return_points(result, viewer)
+    return result
 
 
-@register_function(menu="Points > Sample from surface using Poisson disk (open3d, nppas)")
-def sample_points_poisson_disk(surface:SurfaceData, number_of_points: int = 500, init_factor: float = 5, viewer:napari.Viewer=None):
+@register_function(menu="Points > Create points from surface using Poisson disk sampling (open3d, nppas)")
+def sample_points_poisson_disk(surface:SurfaceData, number_of_points: int = 500, init_factor: float = 5, viewer:napari.Viewer=None) -> PointsData:
     """Sample a list of points from a surface using the Poisson disk algorithm
 
     Parameters
@@ -301,11 +293,11 @@ def sample_points_poisson_disk(surface:SurfaceData, number_of_points: int = 500,
     point_cloud = mesh_in.sample_points_poisson_disk(number_of_points=number_of_points, init_factor=init_factor)
 
     result = to_numpy(point_cloud.points)
-    return return_points(result, viewer)
+    return result
 
 
 @register_function(menu="Points > Down-sample (open3d, nppas)")
-def voxel_down_sample(points_data:PointsData, voxel_size: float = 5, viewer:napari.Viewer=None):
+def voxel_down_sample(points_data:PointsData, voxel_size: float = 5, viewer:napari.Viewer=None) -> PointsData:
     """Removes points from a point cloud so that the remaining points lie within a grid of
     defined voxel size.
 
@@ -316,33 +308,34 @@ def voxel_down_sample(points_data:PointsData, voxel_size: float = 5, viewer:napa
     new_point_cloud = point_cloud.voxel_down_sample(voxel_size)
 
     result = to_numpy(new_point_cloud.points)
-    return return_points(result, viewer)
+    return result
 
 
 @register_function(menu="Points > Points to labels (nppas)")
-@register_function(menu="Segmentation / labeling > Points to labels (nppas)")
+@register_function(menu="Segmentation / labeling > Create labels from points (nppas)")
 @time_slicer
-def points_to_labels(points_data:PointsData, as_large_as_image:LayerData, viewer:napari.Viewer=None) -> LabelsData:
-    """Mark single pixels in an zero-image if there is a point in a given point list.
+def points_to_labels(points_data:PointsData, as_large_as_image:ImageData, viewer:napari.Viewer=None) -> LabelsData:
+    """Mark single pixels in a zero-value pixel image if there is a point in a given point list.
     Point with index 0 in the list will get pixel intensity 1.
     If there are multiple points where the rounded coordinate is within the same pixel,
     some will be overwritten. There is no constraint which will be overwritten.
 
     Parameters
     ----------
-    surface:napari.types.SurfaceData
-    as_large_as_image:napari.types.LayerData
+    points_data:napari.types.PointsData
+    as_large_as_image:napari.types.ImageData
         An image to specify the size of the output image. This image will not be overwritten.
     """
 
-    # labels_stack, _ = np.histogramdd(np.asarray(points_data), bins=as_large_as_image.shape, range=[(0, x) for x in as_large_as_image.shape])
-    # labels_stack = labels_stack.astype(int)
-
-    # labels_stack[points_data] = np.arange(len(points_data))
-
     labels_stack = np.zeros(as_large_as_image.shape, dtype=int)
     for i, p in enumerate(points_data):
-        labels_stack[int(p[0] + 0.5), int(p[1] + 0.5), int(p[2] + 0.5)] = i + 1
+        if len(labels_stack.shape) == 3:
+            labels_stack[int(p[0] + 0.5), int(p[1] + 0.5), int(p[2] + 0.5)] = i + 1
+        elif len(labels_stack.shape) == 2:
+            labels_stack[int(p[0] + 0.5), int(p[1] + 0.5)] = i + 1
+        else:
+            raise NotImplementedError("Points to labels only supports 2D and 3D data")
+            break
 
     return labels_stack
 
@@ -366,7 +359,7 @@ def points_to_convex_hull_surface(points_data:PointsData) -> SurfaceData:
     return to_surface(mesh_out)
 
 
-@register_function(menu="Surfaces > Surface from points (alpha-shape, open3d, nppas)")
+@register_function(menu="Surfaces > Create surface from points (alpha-shape, open3d, nppas)")
 def surface_from_point_cloud_alpha_shape(points_data:PointsData, alpha:float = 5) -> SurfaceData:
     """Turn point into a surface using alpha shapes
 
@@ -385,7 +378,7 @@ def surface_from_point_cloud_alpha_shape(points_data:PointsData, alpha:float = 5
     return to_surface(mesh)
 
 
-@register_function(menu="Surfaces > Surface from points (ball-pivoting, open3d, nppas)")
+@register_function(menu="Surfaces > Create surface from points (ball-pivoting, open3d, nppas)")
 def surface_from_point_cloud_ball_pivoting(points_data:PointsData, radius: float = 5, delta_radius=0) -> SurfaceData:
     """Turn point into a surface using ball pivoting
 
@@ -417,7 +410,7 @@ def surface_from_point_cloud_ball_pivoting(points_data:PointsData, radius: float
     return to_surface(mesh)
 
 
-@register_function(menu="Surfaces > Any label to surface (marching cubes, scikit-image, nppas)")
+@register_function(menu="Surfaces > Create surface from any label (marching cubes, scikit-image, nppas)")
 @time_slicer
 def label_to_surface(labels: LabelsData, label_id: int = 1) -> SurfaceData:
     """
@@ -437,7 +430,7 @@ def label_to_surface(labels: LabelsData, label_id: int = 1) -> SurfaceData:
     return (vertices, faces, values)
 
 
-@register_function(menu="Surfaces > Largest label to surface (marching cubes, scikit-image, nppas)")
+@register_function(menu="Surfaces > Create surface from largest label (marching cubes, scikit-image, nppas)")
 @time_slicer
 def largest_label_to_surface(labels: LabelsData) -> SurfaceData:
     """
@@ -477,3 +470,23 @@ def fill_holes(surface: SurfaceData, size_limit: float = 100) -> SurfaceData:
     mesh.fillHoles(size=size_limit)
 
     return (mesh.points(), np.asarray(mesh.faces()))
+
+@register_function(menu = "Surfaces > Scale surface (isotropic, nppas)",
+                   scale_factor={'min':0.01, 'max':100000})
+def isotropic_scale_surface(surface:SurfaceData, scale_factor:float = 1) -> SurfaceData:
+    """
+    Scales a surface with a given factor.
+
+    Parameters
+    ----------
+    surface
+    scale_factor
+
+    Returns
+    -------
+    surface
+    """
+    result = list(surface)
+    result[0] = result[0] * scale_factor
+    return tuple(result)
+
